/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: pkinitReply.c
*    Creation Date: 27 Oct 2000 (v0.00)
*    VSS Info:
*        $Revision: 1.7 $
*        $Date: 2004/12/03 20:08:54 $
*
****************************************************************************
*    Description:
*
*       Implementation of packetCable PKINIT reply processing
*
****************************************************************************/

#include "pkinitReply.h"
#ifndef WIN32
    #include "cmmtaint.h"
#endif

#include <KerbStatus.h>

const int        SUBJECT_ALT_NAME_OID[7] = {1,3,6,1,5,2,2};
const int        SUBJECT_ALT_NAME_OID_LENGTH = 7;

/* service provider certificate from pkinit.c */
extern unsigned char gtelrootcert[MAX_CERTIFICATE_LENGTH];
extern int gtelrootcert_len;


void make_cert_chain(
                    unsigned char **cert_chain,
                    unsigned long *cert_chain_len,
                    const unsigned char *cert1,
                    const unsigned long cert1_len,
                    const unsigned char *cert2,
                    const unsigned long cert2_len)
{
    int i;

    *cert_chain = (unsigned char *)krbAllocBuf(cert1_len+cert2_len);
    *cert_chain_len = cert1_len+cert2_len;

    memcpy(*cert_chain, cert1, cert1_len);
    for ( i=0;i<cert2_len;i++ )
    {
        (*cert_chain)[i+cert1_len] = cert2[i];
    }
}


/*
*****************************************************************************
** FUNCTION:     secRetrieveKdcCertSubjAltName
**
** PURPOSE:      to verify subjectAltName on the kdc Cert 
**
** PARAMETERS:   cert
**               cert_len
**               the realm from subjectAltName retrieved from certificate
**
** RETURNS:      0 if success, 1 otherwise
**
** NOTE:
*****************************************************************************
*/
krb5_error_code secRetrieveKdcCertSubjAltName(
                                             unsigned char       *cert,
                                             unsigned long       cert_len,
                                             unsigned char       *subjAltNameRealm)
{
    unsigned char          alt_name[256];
    unsigned long          alt_name_len;
    unsigned char          alt_oid[256];
    unsigned long          alt_oid_len;
    unsigned long          oid_array[16];
    int                    olen;
    krb5_data              subj_alt_data;
    krb5_principal         subj_alt_principal;
    int                    i;

    if ( SecLibGetCertIface()->secCertGetSubjectAltNameExt(
                                                          cert,
                                                          cert_len,
                                                          alt_oid,
                                                          &alt_oid_len,
                                                          alt_name, 
                                                          &alt_name_len ) )
    {
        LOG(("ERROR: KDC Cert Bad Could not decode SubjectAltName."));
        return 1;
    }

    //PR8123: Changes so CH and MTA can share the Kerberos ParseLib.a library.
    // This is the same function, just renamed.
    //asn_parse_objid_k(alt_oid, &alt_oid_len, oid_array, &olen);
    mta_asn_parse_objid(alt_oid, &alt_oid_len, oid_array, &olen);
    if ( compare_oid(SUBJECT_ALT_NAME_OID, SUBJECT_ALT_NAME_OID_LENGTH,
                     oid_array, olen) )
    {
        LOG(("ERROR: KDC Cert Bad oid for SubjectAltName."));
        return 1;
    }

    subj_alt_data.data = alt_name;
    subj_alt_data.length = alt_name_len;

    decode_subj_alt_name(&subj_alt_data, &subj_alt_principal);
    subj_alt_principal->realm.data[subj_alt_principal->realm.length] = '\0';
    subj_alt_principal->data[1].data[subj_alt_principal->data[1].length] = '\0';
    subj_alt_principal->data[0].data[subj_alt_principal->data[0].length] = '\0';
    LOG(("Decoded KDC Cert SubjectAltName: realm = %s.", subj_alt_principal->realm.data));
    LOG(("Decoded KDC Cert SubjectAltName: %d name components.", 
         subj_alt_principal->length));
    for ( i=0;i<subj_alt_principal->length;i++ )
    {
        LOG(("Name[%d] = %s.", i, subj_alt_principal->data[i].data));
    }

    strcpy(subjAltNameRealm, subj_alt_principal->data[1].data);
    if ( subj_alt_principal )
    {
        krbFreeBuf(subj_alt_principal->realm.data);
        for ( i=0;i<subj_alt_principal->length;i++ )
            krbFreeBuf(subj_alt_principal->data[i].data);
        krbFreeBuf(subj_alt_principal->data);
    }

    return 0;
}


/*
*****************************************************************************
** FUNCTION:     secVerifyASRepModulus
**
** PURPOSE:      to verify modulus length of all certificates  
**
** PARAMETERS:   out_pk - the pkinit reply structure
**
** RETURNS:      0 if success, 1 otherwise
**
** NOTE:
*****************************************************************************
*/
krb5_error_code secVerifyASRepModulus(
                                     pkinit_reply       *out_pk)
{
    RSA_key                *key;
    unsigned long          pubKeyId;

    ALLOC_RSA_KEY( key );
    if ( SecLibGetCertIface()->secCertGetPublicKey(
                                                  out_pk->dh_signed_data.signed_data.certificates.certs[0].cert,
                                                  out_pk->dh_signed_data.signed_data.certificates.certs[0].cert_len,
                                                  key, 
                                                  &pubKeyId ) )
    {
        FREE_RSA_KEY( key,1 );
        return 1;
    }
    if ( (key->modBufLen != 128) &&
         (key->modBufLen != 192) &&
         (key->modBufLen != 256) )
    {
        LOG(("ERROR: KDC Cert has bad Modulus length %d.", key->modBufLen));
        FREE_RSA_KEY( key,1 );
        return 1;
    }

    if ( out_pk->dh_signed_data.signed_data.certificates.length == 2 )
    {
        /* only SP cert */
        if ( SecLibGetCertIface()->secCertGetPublicKey(
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
                                                      key, 
                                                      &pubKeyId ) )
        {
            FREE_RSA_KEY( key,1 );
            return 1;
        }
        if ( key->modBufLen != 256 )
        {
            LOG(("ERROR: Service Provider Cert has bad Modulus length %d.", key->modBufLen));
            FREE_RSA_KEY( key,1 );
            return 1;
        }
        FREE_RSA_KEY( key,1 );
        return 0;
    }
    if ( out_pk->dh_signed_data.signed_data.certificates.length == 3 )
    {
        /* SP and LocSys certs */
        if ( SecLibGetCertIface()->secCertGetPublicKey(
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
                                                      key, 
                                                      &pubKeyId ) )
        {
            FREE_RSA_KEY( key,1 );
            return 1;
        }
        if ( (key->modBufLen != 128) &&
             (key->modBufLen != 192) &&
             (key->modBufLen != 256) )
        {
            LOG(("ERROR: Local SYS Cert has bad Modulus length %d.", key->modBufLen));
            FREE_RSA_KEY( key,1 );
            return 1;
        }
        /* SP and LocSys certs */
        if ( SecLibGetCertIface()->secCertGetPublicKey(
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[2].cert,
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[2].cert_len,
                                                      key, 
                                                      &pubKeyId ) )
        {
            FREE_RSA_KEY( key,1 );
            return 1;
        }
        if ( key->modBufLen != 256 )
        {
            LOG(("ERROR: Service Provider Cert has bad Modulus length %d.", key->modBufLen));
            FREE_RSA_KEY( key,1 );
            return 1;
        }
    }
    FREE_RSA_KEY( key,1 );
    return 0;
}

/*
*****************************************************************************
** FUNCTION:     check_pkinit_reply
**
** PURPOSE:      check pkinit reply by checking the certificate and signature
**
** PARAMETERS:   out_pk - the pkinit reply structure
**               realm - 
**
** RETURNS:      success or error code
**
** NOTE:
**      
**
*****************************************************************************
*/
krb5_error_code check_pkinit_reply(
                                  pkinit_reply       *out_pk, 
                                  unsigned char      *realm,
                                  int                app_flag,
                                  unsigned char      *subjAltNameRealm)
{
    unsigned char          errorBuffer[256];
    int                    errBufLen=256;
    unsigned long status=OK;
    RSA_key *key;
    unsigned long pubKeyId;
    unsigned char *cert_chain;
    unsigned long cert_chain_len = 0;
    unsigned char *cert_chain1;
    unsigned long cert_chain_len1;
    unsigned long          subject_name_len;
    unsigned char          subject_name[256];
    unsigned char          local_subject_name[256];
    unsigned long          local_subject_name_len;
    unsigned char          local_org_name[256];
    unsigned long          local_org_name_len;
    unsigned long          org_name_len;
    unsigned char          org_name[256];
    unsigned char          *mib_org_name;
    int                    tmp;

    /* IMPORTANT NOTE: AS reply has the following order in certs: 
     * cert[0] is kdc cert
     * cert[1] is SP cert
     * cert[2] is LocSys cert
     * This is just by luck, need to decode and see which is which 
     * */

    /* verify all Modulus lengths first */
    if ( secVerifyASRepModulus(out_pk) )
    {
        return KDC_ERR_KDC_NOT_TRUSTED;
    }

#ifndef DEMO
    if ( !cmNvGetMtaPKTCCompDisable() )
    {
#endif

        /* verify Subject Alt Name extension of the KDc certificate */
        if ( secRetrieveKdcCertSubjAltName(
                                          out_pk->dh_signed_data.signed_data.certificates.certs[0].cert,
                                          out_pk->dh_signed_data.signed_data.certificates.certs[0].cert_len,
                                          subjAltNameRealm) )
        {
            return KDC_ERR_KDC_NOT_TRUSTED;
        }
#ifndef DEMO
    }
#endif


    /* make a chain with telephony root and Service Provider certificates 
     * After decoding certs[1] is the service provider cert and certs[0] 
     * is the TGS certifiacte.
     * If there is a local sys certificate, it will be certs[1] and service
     * provider cert would be cert[2]. TGS certificate is always cert[0].
     * */
    if ( out_pk->dh_signed_data.signed_data.certificates.length == 2 )
    {
        LIBDEBUG(" check_pkinit_reply: Processing 2 certificates in AS Reply\n");
        make_cert_chain(
                       &cert_chain, 
                       &cert_chain_len,
                       gtelrootcert, 
                       gtelrootcert_len, 
                       out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
                       out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len);
    }
    else if ( out_pk->dh_signed_data.signed_data.certificates.length == 3 )
    {
        LIBDEBUG(" check_pkinit_reply: Processing 3 certificates in AS Reply\n");
        /* there is a local sys operator certificate */
        make_cert_chain(
                       &cert_chain1, 
                       &cert_chain_len1,
                       gtelrootcert, 
                       gtelrootcert_len, 
                       out_pk->dh_signed_data.signed_data.certificates.certs[2].cert,
                       out_pk->dh_signed_data.signed_data.certificates.certs[2].cert_len); 

        make_cert_chain(
                       &cert_chain, 
                       &cert_chain_len,
                       cert_chain1, 
                       cert_chain_len1, 
                       out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
                       out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len);

        krbFreeBuf(cert_chain1);

    }

    else
    {
        LIBDEBUG(" check_pkinit_reply: WRONG NUMBER OF CERTS in AS Reply\n");
        /* not enough or too many certificates */
        return KDC_ERR_KDC_NOT_TRUSTED;
    }

#ifndef DEMO
    /* verify the certificate chain only if CM part exists and
     * we have the right time. This call will fail in standalone mode
     * becuase the time is not set properly on the board */
    if ( ! cmNvIsStandaloneEnabled() )
    {
#endif

        /* verify the certificate using the service provider certificate */ 
        status = SecLibGetCertIface()->secCertValidate(
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[0].cert, 
                                                      out_pk->dh_signed_data.signed_data.certificates.certs[0].cert_len,
                                                      cert_chain_len,
                                                      cert_chain,
                                                      errorBuffer,
                                                      errBufLen );
        if ( ktracelevel > KTRACE_ERROR )
        {

            if ( status != OK )
            {
                printf("Result of Certificate Verification: ");
                printf("Code = %d, Text: %s\n", status, errorBuffer);
            }
            else
            {
                printf("Result of Certificate Verification: ");
                printf("status = OK Code = %d\n", status);
            }
        }

#ifndef DEMO
    }
    else
    {
        printf("Telephony Certificate Verification Not Preformed for StandAlone MTA.\n");
    }
#endif

    if ( cert_chain_len )
        krbFreeBuf(cert_chain);


    switch ( status )
    {
    case OK:
        break;

    case BAD_DATA:
        return KDC_ERR_KDC_NOT_TRUSTED;

    case CA_INVALID:
        return KDC_ERR_INVALID_CERTIFICATE;

    case CERT_EXPIRED:
        return KDC_ERR_INVALID_CERTIFICATE;

    default:
        return KDC_ERR_CANT_VERIFY_CERTIFICATE;
    }


    /* verify signature */
    ALLOC_RSA_KEY( key );

    if ( SecLibGetCertIface()->secCertGetPublicKey(
                                                  out_pk->dh_signed_data.signed_data.certificates.certs[0].cert, 
                                                  out_pk->dh_signed_data.signed_data.certificates.certs[0].cert_len,
                                                  key, 
                                                  &pubKeyId ) )
    {
        return KDC_ERR_KDC_NOT_TRUSTED;
    }

    status = SecLibGetCryptoIface()->secCryptoSignVerify(
                                                        out_pk->dh_signed_data.signed_data.encap_content_info.contents,
                                                        out_pk->dh_signed_data.signed_data.encap_content_info.length,
                                                        out_pk->dh_signed_data.signed_data.signer_infos.signer_info.signature.signature_val,
                                                        out_pk->dh_signed_data.signed_data.signer_infos.signer_info.signature.signature_len,
                                                        key );

    FREE_RSA_KEY( key,1 );
    if ( status )
    {
        return KDC_ERR_INVALID_SIG;
    }

//   /* Save service provider name for config file */
//   if ( app_flag == APPLICATION_PROV )
//   {
//      if ( out_pk->dh_signed_data.signed_data.certificates.length == 2 )
//      {
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
//               subject_name,
//               &subject_name_len);
//      }
//      else
//      {
//         /* 3 certificates in AS reply */
//         /* Check orgName in certificate against the one in MIB */
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[2].cert,
//               out_pk->dh_signed_data.signed_data.certificates.certs[2].cert_len,
//               subject_name,
//               &subject_name_len);
//
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
//               local_subject_name,
//               &local_subject_name_len);
//         printf("Local System Certificate Present, SubjectName: %s.\n", local_subject_name);
//      }
//      if ( SecLibGetCertIface()->secCertGetOrgName( 
//               subject_name, subject_name_len, org_name, &org_name_len))
//         return KDC_ERR_KDC_NOT_TRUSTED;
//      
//      printf("Service provider certificate, SubjectName: %s.\n", org_name);
//      secSetProvServiceProviderName(org_name);
//   }
//
//   if ( app_flag == APPLICATION_CMS )
//  {
//      if ( out_pk->dh_signed_data.signed_data.certificates.length == 2 )
//      {
//         /* Check orgName in certificate against the one in MIB */
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
//              out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
//               subject_name,
//               &subject_name_len);
//      }
//	else
//	{
//         /* 3 certificates in AS reply */
//         /* Check orgName in certificate against the one in MIB */
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[2].cert,
//               out_pk->dh_signed_data.signed_data.certificates.certs[2].cert_len,
//               subject_name,
//               &subject_name_len);
//
//         SecLibGetCertIface()->secCertGetSubjectName(
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert,
//               out_pk->dh_signed_data.signed_data.certificates.certs[1].cert_len,
//               local_subject_name,
//               &local_subject_name_len);
//         printf("Local System Certificate Present, SubjectName: %s.\n", local_subject_name);
//      }
//
//      if ( SecLibGetCertIface()->secCertGetOrgName( 
//               local_subject_name, local_subject_name_len, local_org_name, &local_org_name_len))
//         return KDC_ERR_KDC_NOT_TRUSTED;
//      }
//
//      if ( SecLibGetCertIface()->secCertGetOrgName( 
//               subject_name, subject_name_len, org_name, &org_name_len))
//         return KDC_ERR_KDC_NOT_TRUSTED;
//
#ifndef DEMO
    if ( mibAccessSec(
                     MIBITEM(SEC, realmOrgName),
                     NULL,
                     realm,
                     NULL,
                     &tmp,
                     (void*)&mib_org_name,
                     MIB_ITEM_GET) )
    {
        LOG(("MIB VALUE GET realmOrgName failed for realm %s.", realm));
        return KDC_ERR_KDC_NOT_TRUSTED;
    }

    printf("OrgName from Service Provider certificate is %s, from MIB is %s.\n", org_name, mib_org_name);
    if ( strcmp(org_name, mib_org_name ) )
    {
        LIBDEBUG ("ERROR: For realm %s, service provider Certificate OrgName does not match MIB orgName.\n", realm);
        return KDC_ERR_KDC_NOT_TRUSTED;
    }
#endif
//   }

    return KDC_ERR_NONE;
}   
