/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: os.c
*    Creation Date: 28 June 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.2 $
*        $Date: 2002/12/23 15:13:13 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file contains a VxWorks
*     specific implementation of the BCMOS API.
*
****************************************************************************/

#if CH_NEEDED
#include <hausenv.h>
#include <string.h>
#include <bcmos.h>
#include "../osdef.h"
#include "socket.h"
#include "dns.h"

/* Operating system specific includes */
#include <vxWorks.h>
#include <taskLib.h>
#endif

#include "vxwrkdef.h"


/*
*****************************************************************************
** TYPE DEFINITIONS and CONSTANTS
*****************************************************************************
*/

/* Definitions for task options */
///#define OS_DEFAULT_TASKFLAGS     0



/*****************************************************************************
** FUNCTION:   osInit
**
** PURPOSE:    initialize any os specific variables
**
** PARAMETERS: priorities - structure that contains the absolute priority
**                          levels associated with each class of MTA task.
**
** RETURNS:    OS_STATUS_OK
*****************************************************************************/
OS_STATUS osInit( void )
{
   #if CH_NEEDED
   OS_STATUS   status;

   /* Initialize socket interface */
   if (( status = socketInit() ) != OS_STATUS_OK )
   {
      return( status );
   }

   /* Initialize socket sets */
   if (( status = socketSetInit() ) != OS_STATUS_OK )
   {
      return( status );
   }

   /* Initialize DNS state */
   if (( status = dnsInit() ) != OS_STATUS_OK )
   {
      return( status );
   }
#endif
   return( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   checkStatus
**
** PURPOSE:    This function takes the status returned by an OS-specific API
**             function call. It maps this to a generic BcmOS return status, and
**             logs any errors.
**
** PARAMETERS: status   - operating system specific status returned from system call
**             fnc      - name of calling function, used for error LOG statements
**
** RETURNS:    Operating system independent status value of type OS_STATUS
**             (that corresponds to 'status') . BcmOS public interface functions
**             should return OS independent status values. Native OS status
**             values should be encapsulated; however, they may be LOGed for
**             debug purposes.
**
** NOTES:      This function is used to validate all OS-specific API calls invoked
**             by the BcmOS layer that deal with tasks, mutexes, semaphores, events,
**             message queues... However, not all OS API calls return consistent
**             status values/error codes. Therefore, the following assumptions have
**             been made:
**
**             PSOS:    an error is indicated by a non-zero 'status' value
**             VXWORKS: an error is indicated by a 'status' value of ERROR
**             WIN32:   an error is indicated by a 'status' value of 0
**
**             These assumptions should handle most OS-specific API calls. However, it
**             is the responsibility of the caller to massage the input parameters to conform
**             to these assumptions for any OS API calls that do not follow these rules.
**
*****************************************************************************/
OS_STATUS checkStatus( NATIVE_STATUS status, char *fnc )
{
   int      err;
   BOOL     bTimeout;
   BOOL     bErrStatus;

   /* Get OS-dependent error value and status... */

   /* Most VxWorks functions return ERROR when they detect an error (or NULL in
    * the case of functions returning pointers). In addition, they set an error
    * status that elaborates upon the nature of the error. This can be retrieved
    * from the errno variable. */
   err = errno;
   bTimeout = ( err == S_objLib_OBJ_TIMEOUT );
   bErrStatus = ( status == ERROR );

   /* Log errors (if any), and return OS-independent status value */
   if ( bTimeout )
   {
      return ( OS_STATUS_TIMEOUT );
   }
   else if ( bErrStatus )
   {
      LOG(( "Error returned by %s - 0x%x\n", fnc, err ));
      return ( OS_STATUS_ERR );
   }

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsSleep
**
** PURPOSE:    Sleep for a specified number of ticks
**
** PARAMETERS: sleeptime - the time to sleep for (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
*****************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsSleep( OS_TICKS sleeptime )
{
   NATIVE_STATUS  status;

   /* Convert to OS-specific tick value */
   sleeptime = sleeptime / OS_TICK_SCALE_FACTOR;

   /* Delay the specified time. */
   status = taskDelay((int) sleeptime );

   /* Error checking */
   return ( checkStatus( status, "bcmOsSleep" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskCreate
**
** PURPOSE:    Create a task.
**
** PARAMETERS: name        - name of the of the task
**             stackSize   - task stack size
**             priority    - task priority. If <priority> is an OS_TASK_CLASS
**                           enumerated type, the real OS priority will be set
**                           according to the OS_THREAD_CLASS_PRIO structure
**                           passed to bcmOsInit. If <priority> is a positive
**                           number, (all OS_TASK_CLASS types are negative),
**                           this number will be used as the real OS priority
**             taskentry   - the task entry point
**             argument    - the task argument
**             taskId      - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskCreate( char* name,
                           int stackSize,
                           OS_TASK_CLASS taskClass,
                           void* taskEntry,
                           OS_TASKARG argument,
                           OS_TASKID *taskId )
{
   UINT32 priority;

   ASSERT( name && taskEntry && taskId);

   /*
   ** Spawn the task.
   */
   bcmOsTaskPriority( taskClass, &priority );
   *taskId = (OS_TASKID) taskSpawn( name, priority, OS_DEFAULT_TASKFLAGS,
                                    stackSize, (FUNCPTR)taskEntry, (int) argument, 0,0,0,0,0,0,0,0,0 );

   return( checkStatus( *taskId, "bcmOsTaskCreate" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskDelete
**
** PURPOSE:    deletes a task
**
** PARAMETERS: taskID - task identification
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskDelete( OS_TASKID taskId )
{
   NATIVE_STATUS  status;

   /*
   ** Delete the task
   */
   status = taskDelete( (int) taskId);

   /* Error checking */
   return( checkStatus( status, "bcmOsTaskDelete" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskRestart
**
** PURPOSE:    restarts a task
**
** PARAMETERS: name - name of the task
**             argument - the task arguments
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskRestart( char* name, OS_TASKARG argument )
{
   NATIVE_STATUS  status = ERROR;

   /* Error checking */
   return ( checkStatus( status, "bcmOsTaskRestart" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskIdent
**
** PURPOSE:    obtains the task identifier, given a task name.
**
** PARAMETERS: taskName - name of task
**             taskID   - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskIdent(char* taskName, OS_TASKID *taskId )
{
   ASSERT( taskName && taskId );

   *taskId = (OS_TASKID) taskNameToId (taskName);

   /* Error checking */
   return( checkStatus( *taskId, "bcmOsTaskIdent" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskIdentSelf
**
** PURPOSE:    obtains the task identifier of the calling task
**
** PARAMETERS:
**             taskID   - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskIdentSelf(OS_TASKID *taskId )
{
   ASSERT ( taskId );

   *taskId = (OS_TASKID) taskIdSelf();

   /* Error checking */
   return( checkStatus( *taskId, "bcmOsTaskIdentSelf" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskSuspend
**
** PURPOSE:    Suspends a task
**
** PARAMETERS: taskID - task identifier ( 0 for suicide )
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskSuspend( OS_TASKID taskId )
{
   NATIVE_STATUS  status;

   status = taskSuspend( (int) taskId);

   /* Error checking */
   return( checkStatus( status, "bcmOsTaskSuspend" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskResume
**
** PURPOSE:    Resumes a suspended a task
**
** PARAMETERS: taskID - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskResume( OS_TASKID taskId )
{
   NATIVE_STATUS  status;

   status = taskResume( taskId );

   /* Error checking */
   return( checkStatus( status, "bcmOsTaskResume" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskGetPriority
**
** PURPOSE:    Gets task priority type, which is OS-independent
**
** PARAMETERS: taskID      - task identifier
**             taskClass   - task type (class)
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskGetPriority( OS_TASKID taskId, OS_TASK_CLASS *taskClass )
{
   int            priority;
   NATIVE_STATUS  status;

   ASSERT( taskClass );

   status = taskPriorityGet((int) taskId, &priority );

   /* Map the OS-specific task priority into an OS-independent MTA task class */
   bcmOsTaskClass((UINT32) priority, taskClass );

   /* Error checking */
   return( checkStatus( status, "bcmOsGetPriority" ));
}
#endif

/*****************************************************************************
** FUNCTION:   bcmOsTaskExists
**
** PURPOSE:    Verifies whether a task exists or not
**
** PARAMETERS: taskID   - task identifier to be verified
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#if CH_NEEDED
OS_STATUS bcmOsTaskExists( OS_TASKID taskId )
{
   int            priority;
   NATIVE_STATUS  status;

   status = taskPriorityGet((int) taskId, &priority);

   /* Error checking */
   return ( checkStatus( status, "bcmOsTaskExists" ));
}
#endif
