/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: sem.c
*    Creation Date: 28 June 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:54:41 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file contains the VxWorks
*     specific implementation of semaphores.
*
****************************************************************************/

#include <hausenv.h>
#include <string.h>
#include <bcmos.h>

/* Operating system specific includes */
#include <vxWorks.h>
#include <semLib.h>
#include "vxwrkdef.h"


/*****************************************************************************
** FUNCTION:   bcmOsSemCreate
**
** PURPOSE:    Create a binary semaphore
**
** PARAMETERS: semName - semaphore name
**             semId - pointer to the semaphore handle
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemCreate(char *semName, OS_SEMID *semId)
{
   NATIVE_STATUS  status;

   ASSERT( semName && semId );

   /*
   ** Create a binary semaphore
   */
   *semId = (OS_SEMID) semBCreate( SEM_Q_FIFO, SEM_EMPTY );

   /* semBCreate() returns NULL for an error. This is inconsistent with most other
    * VxWorks API calls which return ERROR for an error. If NULL is returned,
    * change the return code to ERROR so that the same error checking function can
    * be used for all API calls. */
   status = OK;
   if ( *semId == 0 )
   {
      status = ERROR;
   }

   /* Error checking */
   return( checkStatus( status, "bcmOsSemCreate" ));
}


/*****************************************************************************
** FUNCTION:   bcmOsSemTake
**
** PURPOSE:    Take a binary semaphore
**
** PARAMETERS: semId - Handle of the semaphore
**             timeout_ticks - timeout in ticks
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       this function ASSERTs that the operation was successful
******************************************************************************/
OS_STATUS bcmOsSemTake(OS_SEMID semId, OS_TICKS timeout)
{
   NATIVE_STATUS  status;
   int            tickWait;

   ASSERT(semId);

   /* Convert to OS-specific tick value */
   if ( timeout == OS_WAIT_FOREVER )
   {
      /* Wait forever */
      tickWait = WAIT_FOREVER;
   }
   else if ( timeout )
   {
      /* Wait until either semaphore is given, or timeout expires */
      tickWait = timeout / OS_TICK_SCALE_FACTOR;
   }
   else
   {
      /* Non-blocking semaphore take */
      tickWait = NO_WAIT;
   }

   status = semTake((SEM_ID) semId, tickWait );


   /* Error checking */
   return( checkStatus( status, "bcmOsSemTake" ));
}


/*****************************************************************************
** FUNCTION:   bcmOsSemGive
**
** PURPOSE:    Give a semaphore
**
** PARAMETERS: semId - Handle of the semaphore
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemGive( OS_SEMID semId )
{
   NATIVE_STATUS  status;

   ASSERT( semId );

   status = semGive((SEM_ID) semId );

   /* Error checking */
   return( checkStatus( status, "bcmOsSemGive" ));
}


/*****************************************************************************
** FUNCTION:   bcmOsSemDelete
**
** PURPOSE:    deletes a semaphore
**
** PARAMETERS: semID - semaphore identification
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemDelete( OS_SEMID semId )
{
   NATIVE_STATUS  status;

   /*
   ** Delete the task
   */
   status = semDelete( (SEM_ID) semId);

   /* Error checking */
   return( checkStatus( status, "bcmOsSemDelete" ));
}
