/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: sockset.c
*    Creation Date: 28 June 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.2 $
*        $Date: 2002/10/16 13:40:10 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file contains the VxWorks
*     specific implementation of socket sets.
*
****************************************************************************/

#include <hausenv.h>
#include <string.h>
#include <bcmos.h>
#include <bcmosCfg.h>
#include "socket.h"

/* Operating system specific includes */
#include <vxWorks.h>
#include <sockLib.h>
#include <unistd.h>
#include <selectLib.h>


/*
*****************************************************************************
** TYPE DEFINITIONS and CONSTANTS
*****************************************************************************
*/

/* Number of 'sets' in the file descriptor set table */
#define MAX_FD_SETS     OS_CFG_MAX_FD_SETS

/* File descriptor set entry */
typedef struct FDSET
{
   fd_set   fdSet;
   BOOL     bFree;
} FDSET;


/*
*****************************************************************************
** LOCAL VARIABLES
*****************************************************************************
*/

/* File descriptor set table */
static FDSET fdSetTable[ MAX_FD_SETS ];


/*
*****************************************************************************
** LOCAL FUNCTIONS
*****************************************************************************
*/

static OS_STATUS checkSocketSetStatus( NATIVE_SOCKET_STATUS status, char *fnc );


/*****************************************************************************
** FUNCTION:   checkSocketSetStatus
**
** PURPOSE:    This function takes the status returned by an OS-specific socket set
**             API function call. It maps this to a generic BcmOS return status, and
**             logs any errors.
**
** PARAMETERS: status   - operating system specific status returned from system call
**             fnc      - name of calling function, used for error LOG statements
**
** RETURNS:    Operating system independent status value of type OS_STATUS
**             (that corresponds to 'status') . BcmOS public interface functions
**             should return OS independent status values. Native OS status
**             values should be encapsulated; however, they may be LOGed for
**             debug purposes.
**
** NOTES:      This function is used to validate all OS-specific socket-related API
**             calls invoked by the BcmOS layer. However, not all OS API calls return
**             consistent status values/error codes. Therefore, the following assumptions
**             have been made:
**
**             PSOS:    an error is indicated by a 'status' value of -1
**             VXWORKS: an error is indicated by a 'status' value of ERROR
**             WIN32:   an error is indicated by a 'status' value of INVALID_SOCKET
**                      or SOCKET_ERROR
**
**             These assumptions should handle most OS-specific API calls. However, it
**             is the responsibility of the caller to massage the input parameters to conform
**             to these assumptions for any OS API calls that do not follow these rules.
**
*****************************************************************************/
static OS_STATUS checkSocketSetStatus( NATIVE_SOCKET_STATUS status, char *fnc )
{
   int      err;
   BOOL     bErrStatus;
   BOOL     bTimeout;

   /* Get OS-dependent error value and status */
   /* Most VxWorks functions return ERROR when they detect an error (or NULL in
    * the case of functions returning pointers). In addition, they set an error
    * status that elaborates upon the nature of the error. This can be retrieved
    * from the errno variable. */
   err = errno;
   bErrStatus = ( status == ERROR );
   bTimeout = ( status == 0 );

   /* Log errors (if any), and return OS-independent status value */
   if ( bTimeout )
   {
      return ( OS_STATUS_TIMEOUT );
   }
   else if ( bErrStatus )
   {
      LOG(( "Error returned by %s - 0x%x\n", fnc, err ));
      return ( OS_STATUS_ERR );
   }
   return ( OS_STATUS_OK );

}


/*****************************************************************************
** FUNCTION:   socketSetInit
**
** PURPOSE:    Initialize any socket set related interfaces
**
** PARAMETERS: None
**
** RETURNS:    OS_STATUS_OK or OS_STATUS_ERR
*****************************************************************************/
OS_STATUS socketSetInit( void )
{
   int   i;

   /* Clear socket-descriptor set table */
   memset( fdSetTable, 0, sizeof( FDSET ) * MAX_FD_SETS );

   /* Set all socket-descriptor sets to free */
   for ( i = 0; i < MAX_FD_SETS; i++ )
   {
      fdSetTable[i].bFree = TRUE;
   }

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsSelect()
**
** PURPOSE:    This function is used to multiplex I/O requests among multiple sockets.
**             Three sets of socket descriptors may be specified: a set of sockets from
**             which to read, a set to which to write and a set that may have pending
**             exceptional conditions.
**
**             If bcmOsSelect() returns successfully, the three sets indicate which socket
**             descriptors can be read, which can be written to, or which have exceptional
**             conditions pending. A timeout value may be specified.
**
** PARAMETERS: readset     [IN] - Set of sockets from which to read
**             writeset    [IN] - Set of sockets from which to write
**             exceptset   [IN] - Set of sockets that may have an exceptional condition pending
**             timeout     [IN] - Specifies a timeout option. If the timeout is set to 0,
**                                bcmOsSelect() returns immediately. If the timeout is
**                                OS_WAIT_FOREVER, bcmOsSelect() blocks until a descriptor is
**                                selectable.
**             numDesc     [OUT] - On success a nonzero value is returned that indicates the
**                                 number of descriptors on which events have occurred.
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE: Any two of readset, writeset, and exceptset may be OS_FD_SET_NULL.
*****************************************************************************/
OS_STATUS bcmOsSelect( OS_FD_SET readset, OS_FD_SET writeset, OS_FD_SET exceptset, OS_TICKS timeout, int *numDesc )
{

   fd_set   *rd    = (( readset == OS_FD_SET_NULL ) ? NULL : &fdSetTable[ readset ].fdSet );
   fd_set   *wrt   = (( writeset == OS_FD_SET_NULL ) ? NULL : &fdSetTable[ writeset ].fdSet );
   fd_set   *excpt = (( exceptset == OS_FD_SET_NULL ) ? NULL : &fdSetTable[ exceptset ].fdSet );
   struct timeval  timeoutVal;

   timeoutVal.tv_sec = TICKS2SEC( timeout );
   timeoutVal.tv_usec = ( timeout * OS_MSEC_PER_TICK * 1000 ) % 1000000;

   if ( timeout != OS_WAIT_FOREVER )
   {
      *numDesc = select( FD_SETSIZE, rd, wrt, excpt, &timeoutVal );
   }
   else
   {
      *numDesc = select( FD_SETSIZE, rd, wrt, excpt, NULL );
   }

   return ( checkSocketSetStatus( *numDesc, "bcmOsSelect" ));
}


/*****************************************************************************
** FUNCTION:   bcmOsFdSetOpen()
**
** PURPOSE:    Create a new file-descriptor set
**
** PARAMETERS: set [OUT]  - descriptor for created file-descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetOpen( OS_FD_SET *set )
{
   int         i;
   OS_LOCKKEY  lock;
   OS_STATUS   status = OS_STATUS_ERR;

   ASSERT( set );

   /* Concurrency protection */
   //bcmOsLock( &lock );

   /* Find a free fd-set in the fd-set table */
   for ( i = 0; i < MAX_FD_SETS; i++ )
   {
      if ( fdSetTable[i].bFree == TRUE )
      {
         fdSetTable[i].bFree = FALSE;
         *set = i;
         status = OS_STATUS_OK;
         break;
      }
   }

   //bcmOsUnlock( lock );

   if ( status != OS_STATUS_OK )
   {
      LOG(("ERROR: bcmOsFdSetOpen() - No free slots in \"fdSetTable\""));
   }

   return ( status );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdSetClose()
**
** PURPOSE:    Close an existing file-descriptor set
**
** PARAMETERS: set [IN]  - descriptor for file-descriptor set to close
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetClose( OS_FD_SET set )
{
   OS_LOCKKEY  lock;
   OS_STATUS   status;

   ASSERT( set < MAX_FD_SETS );

   /* Concurrency protection */
   //bcmOsLock( &lock );

   /* Free entry in fd-set table */
   status = bcmOsFdZero( set );
   fdSetTable[ set ].bFree = TRUE;

   //bcmOsUnlock( lock );

   return ( status );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdSetCopy()
**
** PURPOSE:    Copy a file-descriptor set
**
** PARAMETERS: dstSet [IN]  - descriptor for file-descriptor set to copy to
**             srcSet [IN]  - descriptor for file-descriptor set to copy from
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetCopy( OS_FD_SET dstSet, OS_FD_SET srcSet )
{
   OS_LOCKKEY  lock;

   ASSERT( dstSet < MAX_FD_SETS );
   ASSERT( srcSet < MAX_FD_SETS );

   /* Concurrency protection */
   //bcmOsLock( &lock );

   fdSetTable[ dstSet ] = fdSetTable[ srcSet ];

   //bcmOsUnlock( lock );

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdSet()
**
** PURPOSE:    Add a socket descriptor to a set. Emulates Berkeley Socket
**             macro FD_SET
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSet( int desc, OS_FD_SET set )
{
   ASSERT( set < MAX_FD_SETS );

   FD_SET( desc, &fdSetTable[ set ].fdSet );

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdClr()
**
** PURPOSE:    Remove a socket descriptor From a set. Emulates Berkeley Socket
**             macro FD_CLR
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdClr( int desc, OS_FD_SET set )
{
   ASSERT( set < MAX_FD_SETS );

   FD_CLR( desc, &fdSetTable[ set ].fdSet );

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdIsSet()
**
** PURPOSE:    Determines if a socket descriptor is a member of a set. Emulates
**             Berkeley Socket macro FD_ISSET
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
 *             isSet [OUT] - indicates whether socket is member of the set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdIsSet( int desc, OS_FD_SET set, BOOL *isSet )
{
   ASSERT( set < MAX_FD_SETS );

   *isSet = FD_ISSET( desc, &fdSetTable[ set ].fdSet );

   return ( OS_STATUS_OK );
}


/*****************************************************************************
** FUNCTION:   bcmOsFdZero()
**
** PURPOSE:    Clears an entire socket descriptor set. Emulates Berkeley Socket
**             macro FD_ZERO
**
** PARAMETERS: set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdZero( OS_FD_SET set )
{
   ASSERT( set < MAX_FD_SETS );

   FD_ZERO( &fdSetTable[ set ].fdSet );

   return ( OS_STATUS_OK );
}
