/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: appos.h
*    Creation Date: 17 September 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.2 $
*        $Date: 2005/08/15 15:05:24 $
*
****************************************************************************
*    Description:
*
*     Broadcom Application-specific Operating System Functions. This file should
*     only contain functions which are necessary to supplement the bcmos
*     functionality to implement application specific purposes.
*
****************************************************************************/

#ifndef _APPOS_H
#define _APPOS_H

#ifdef __cplusplus
extern "C" {
#endif


#include <hausenv.h>
#include <xbcmos.h>      /* AppOs "inherits" from BcmOs, and therefore uses all of
                           its public typedefs and defines */
//#include "trc.h"
#include "bcmosHelper.h"

/*
*****************************************************************************
** TYPE DEFINITIONS and CONSTANTS
*****************************************************************************
*/

/* APPOS API return codes. APPOS "inherits" the BCMOS return codes. In addition,
 * it also defines it's own set of application specific return codes.  */
#define AOS_STATUS_RESET   OS_STATUS_APPLEVEL1


/* APPOS task start modes */
typedef enum
{
   AOS_TASKSTART_RUNNING,
   AOS_TASKSTART_SUSPENDED

} AOS_TASKSTARTMODE;


/* Task init, main, and de-init function prototypes */
typedef void (*AOS_TASK_INIT_FNC)( void );
typedef void (*AOS_TASK_MAIN_FNC)( OS_TASKARG );
typedef void (*AOS_TASK_DEINIT_FNC)( void );

/* Application module init and de-init function prototypes. These must be the
 * same as the task init and de-init prototypes. */
typedef AOS_TASK_INIT_FNC     AOS_APP_INIT_FNC;
typedef AOS_TASK_DEINIT_FNC   AOS_APP_DEINIT_FNC;




/*****************************************************************************
** FUNCTION:   appOsInit
**
** PURPOSE:    Initialize any OS specific variables
**
** PARAMETERS: priorities - structure that contains the absolute priority
**                          levels associated with each class of MTA task.
**
** RETURNS:    OS_STATUS_OK or error code
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsInit( OS_THREAD_CLASS_PRIO *priorities );


/*****************************************************************************
** FUNCTION:   appOsTaskCreateMode
**
** PURPOSE:    Create a task
**
** PARAMETERS: name - name of the task (4 characters max)
**             stackSize - task stack size (bytes)
**             taskClass - task priority class
**             taskInit - task initialization function
**             taskMain - task main function
**             taskDeinit - task de-initialization function
**             argument - the task argument passed to <taskMain>
**             taskId - task identifier
**             bUnInterruptible - indicates if task is uninterruptible.
**                      Uninterruptible tasks do not allocate or use OS resources,
**                      and therefore are not informed of application resets.
**             taskStartMode - Mode to start the task (AOS_TASKSTARTMODE_RUNNING | SUSPENDED)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsTaskCreateMode( char* name,
                               int stackSize,
                               OS_TASK_CLASS taskClass,
                               AOS_TASK_INIT_FNC taskInit,
                               AOS_TASK_MAIN_FNC taskMain,
                               AOS_TASK_DEINIT_FNC taskDeinit,
                               OS_TASKARG argument,
                               OS_TASKID *taskId,
                               BOOL bUnInterruptible,
                               AOS_TASKSTARTMODE taskStartMode );


/*****************************************************************************
** FUNCTION:   appOsTaskCreate
**
** PURPOSE:    Create a task (in RUNNING mode)
**
** PARAMETERS: name - name of the task (4 characters max)
**             stackSize - task stack size (bytes)
**             taskClass - task priority class
**             taskInit - task initialization function
**             taskMain - task main function
**             taskDeinit - task de-initialization function
**             argument - the task argument passed to <taskMain>
**             taskId - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
#define           appOsTaskCreate(nm, sz, cl, i, m, d, arg, id)   \
              appOsTaskCreateMode(nm, sz, cl, i, m, d, arg, id, FALSE, AOS_TASKSTART_RUNNING)


/*****************************************************************************
** FUNCTION:   appOsTaskCreateSuspended
**
** PURPOSE:    Create a task (in SUSPENDED mode)
**
** PARAMETERS: name - name of the task (4 characters max)
**             stackSize - task stack size (bytes)
**             taskClass - task priority class
**             taskInit - task initialization function
**             taskMain - task main function
**             taskDeinit - task de-initialization function
**             argument - the task argument passed to <taskMain>
**             taskId - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       You must call appOsTaskStart() to start the task
******************************************************************************/
#define  appOsTaskCreateSuspended(nm, sz, cl, i, m, d, arg, id)   \
              appOsTaskCreateMode(nm, sz, cl, i, m, d, arg, id, FALSE, AOS_TASKSTART_SUSPENDED)


/*****************************************************************************
** FUNCTION:   appOsTaskCreateUnInterruptible
**
** PURPOSE:    Create an un-interruptible task. Uninterruptible tasks do not
**             allocate or use OS resources, and therefore are not informed
**             of application resets.
**
** PARAMETERS: name - name of the task (4 characters max)
**             stackSize - task stack size (bytes)
**             taskClass - task priority class
**             taskInit - task initialization function
**             taskMain - task main function
**             taskDeinit - task de-initialization function
**             argument - the task argument passed to <taskMain>
**             taskId - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       You must call appOsTaskStart() to start the task
******************************************************************************/
#define  appOsTaskCreateUnInterruptible(nm, sz, cl, i, m, d, arg, id)   \
                    appOsTaskCreateMode(nm, sz, cl, i, m, d, arg, id, TRUE, AOS_TASKSTART_RUNNING)


/*****************************************************************************
** FUNCTION:   appOsTaskDelete
**
** PURPOSE:    Deletes a task
**
** PARAMETERS: taskId - task identification
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsTaskDelete( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   appOsTaskStart
**
** PURPOSE:    Start a task which was created in suspended mode
**
** PARAMETERS: taskId - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsTaskStart( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   appOsSleep
**
** PURPOSE:    Sleep for a specified number of ticks
**
** PARAMETERS: sleeptime - the time to sleep for (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsSleep( OS_TICKS sleeptime );


/*****************************************************************************
** FUNCTION:   appOsTaskSuspend
**
** PURPOSE:    Suspends a task
**
** PARAMETERS: taskId - task identifier  ( 0 for suicide )
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsTaskSuspend( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   appOsTaskResume
**
** PURPOSE:    Resumes a suspended a task
**
** PARAMETERS: taskId - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsTaskResume( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   appOsSemTake
**
** PURPOSE:    Take a semaphore
**
** PARAMETERS: semId - Handle of the semaphore
**             timeout_ticks - timeout in ticks
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsSemTake(OS_SEMID semId, OS_TICKS timeout);


/*****************************************************************************
** FUNCTION:   appOsMutexLock
**
** PURPOSE:    Lock a mutex
**
** PARAMETERS: mutexId        - Handle of the mutex
**             timeout_ticks  - timeout in ticks
**
** RETURNS:    OS_STATUS_OK   - success
**             error code     - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsMutexLock(OS_MUTEXID mutexId, OS_TICKS timeout);


/*****************************************************************************
** FUNCTION:   appOsEventReceive
**
** PURPOSE:    Wait for an event to be sent to the calling task
**
** PARAMETERS: event - event to wait for
**             timeout - event timeout delay (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsEventReceive( OS_EVENT *event, OS_TICKS timeout );


/*****************************************************************************
** FUNCTION:   appOsQueueReceive
**
** PURPOSE:    Receives a message from the specified message queue
**
** PARAMETERS: qid      - Message Queue ID
**             timeout  - Timeout in clock ticks
**             qMsg     - Message Buffer
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       If timeout is 0, bcmOsQueueReceive() will be non-blocking,
**             i.e. it will return even if there is no available message
**             in the queue. If timeout is not 0, then this function will
**             block until a message is posted to the queue or the timeout
**             occurs, which ever comes first.
**
******************************************************************************/
OS_STATUS appOsQueueReceive( OS_QID qid, OS_TICKS timeout, OS_QMSG *qMsg );


/*****************************************************************************
** FUNCTION:   appOsSelect
**
** PURPOSE:    This function is used to multiplex I/O requests among multiple
**             sockets. Three sets of socket descriptors may be specified: a
**             set of sockets from which to read, a set to which to write and
**             a set that may have pending exceptional conditions.
**
**             If bcmOsSelect returns successfully, the three sets indicate
**             which socket descriptors can be read, which can be written to,
**             or which have exceptional conditions pending. A timeout value
**             may be specified.
**
** PARAMETERS: readset     [IN] - Set of sockets from which to read
**             writeset    [IN] - Set of sockets from which to write
**             exceptset   [IN] - Set of sockets that may have an exceptional
**                                condition pending
**             timeout     [IN] - Specifies a timeout option. If the timeout is
**                                set to 0, bcmOsSelect() returns immediately. If
**                                the timeout is OS_WAIT_FOREVER, bcmOsSelect()
**                                blocks until a descriptor is selectable.
**             numDesc     [OUT] - On success a nonzero value is returned that
**                                 indicates the number of descriptors on which
**                                 events have occurred.
**
** RETURNS:   OS_STATUS_OK or error code
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsSelect( OS_FD_SET readset,
                       OS_FD_SET writeset,
                       OS_FD_SET exceptset,
                       OS_TICKS  timeout,
                       int       *numDesc );


/*****************************************************************************
** FUNCTION:   appOsSocketReceiveFrom
**
** PURPOSE:    Receive a message from a socket
**
** PARAMETERS:
**             sd            - socket to receive from
**             buf           - pointer to data buffer
**             buflen        - length of buffer
**             flags         - flags to underlying protocols
**             ifno          - specifies outgoing interface for message
**                             (0 for default interface)
**             from          - where to copy sender's address
**             fromLen       - pointer to size of "from" buffer
**             numBytesRec   - number of bytes received
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
******************************************************************************/
OS_STATUS appOsSocketReceiveFrom( int           sd,
                                  char          *buf,
                                  int           buflen,
                                  int           flags,
                                  long          ifno,
                                  OS_SOCKADDR   *from,
                                  int           *fromLen,
                                  int           *numBytesRec);


/*****************************************************************************
** FUNCTION:   appOsTftpReadFile
**
** PURPOSE:    This function reads the specified file from the specified TFTP
**             server, placing the file data in the buffer.
**             Up to "bufferSize" bytes of the file will be read.
**
** PARAMETERS: serverIpAddress (I)  - the IP address of the TFTP server.
**             pFilename       (I)  - the fully qualified name of the file
**                                    to be read.
**             ifno            (I)  - IP Iface # over which the TFTP is performed
**             pBuffer         (O)  - buffer with retrieved data.
**             bufferSize      (I)  - Size of the buffer.
**             pBytesRead      (O)  - the number of bytes actually read.
**             abortFunc       (I)  - function to call to check if file read should
**                                    be aborted mid-stream.
**
** RETURNS:    OS_STATUS_OK   - success
**             error code     - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsTftpReadFile(  IPADDR serverIpAddress,
                              char *pFilename,
                              UINT32 ifno,
                              char *pBuffer,
                              UINT32 bufferSize,
                              UINT32 *pBytesRead,
                              OS_ABORT_FNC abortFunc );


/****************************************************************************
** FUNCTION:   appOsTaskResetNotify
**
** PURPOSE:    This is a blocking call that informs a task of a reset
**             and waits for the task to ACK the reset.
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsTaskResetNotify( OS_TASKID taskid );


/****************************************************************************
** FUNCTION:   appOsTaskIsResetPending
**
** PURPOSE:    Checks if the calling task has been notified of a pending reset
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
BOOL appOsTaskIsResetPending( void );


/*****************************************************************************
** FUNCTION:   appOsTaskResetAck
**
** PURPOSE:    This function should be invoked by tasks to acknowledge that
**             they have been notified that a reset is pending. Calling this
**             function will cause the calling task to block until it is
**             placed into the reset-active state.
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsTaskResetAck( void );


/*****************************************************************************
** FUNCTION:   appOsTaskProceedToShutdown
**
** PURPOSE:    This function will change the state of a task to 'shutdown'
**             (from reset-ack). Tasks are blocked in the reset-ack state.
**             Changing the task's state to 'shutdown' will cause the task to
**             resume operation. In this state, tasks should perform operations
**             required for task shutdown before exiting their main task function.
**
** PARAMETERS: taskid - change this task to reset active
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsTaskProceedToShutdown( OS_TASKID taskid );


/*****************************************************************************
** FUNCTION:   appOsTaskWaitForDone
**
** PURPOSE:    This function will block until the specified task is "done".
**             A task is considered done once it has exited from it's main
**             task function and completed execution of the task de-init
**             function.
**
** PARAMETERS: taskid - wait until this task is done
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
OS_STATUS appOsTaskWaitForDone( OS_TASKID taskid );


/*****************************************************************************
** FUNCTION:   appOsStartApp
**
** PURPOSE:    This function is responsible for spawning the application. The
**             application initialization function will be invoked asynchronously.
**             This function should spawn all necessary application tasks,
**             allocate resources, and init local storage. The application
**             de-initialization function will be invoked when an application
**             reset is initiated. This function should wait for tasks to
**             complete the reset, and de-allocate resources.
**
** PARAMETERS: appInit - application initialization function
**             appDeinit - application deinitialization function
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
*****************************************************************************/
void appOsStartApp( AOS_APP_INIT_FNC appInit, AOS_APP_DEINIT_FNC appDeinit );


/*****************************************************************************
** FUNCTION:   appOsResetApp
**
** PURPOSE:    Initiate an application reset
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       The application reset is performed asynchronously, this function
**             returns immediately
*****************************************************************************/
void appOsResetApp( void );


#ifdef __cplusplus
    }
#endif

#endif  /* _APPOS_H  */


